package  juice.example;

import   jam.vector.*;
import   jam.JamObject;

/**
 * An RnSpc is a vector space of arrays (of some dimension) of real numbers.
 * <BR><!--#include file="./source.inc"-->
 * 
 * @see         RnVec, RnOp
 * @author      Joel Schroeder
 */
public class RnSpc implements Space {
  protected int[] shape;
  protected int   n;
  
  protected RnSpc() {}

  /** copy constructor */
  protected RnSpc(RnSpc src) {
    this.n = src.n;
    this.shape = new int[src.shape.length];
    for (int i = 0; i < this.shape.length; i++)
      this.shape[i] = src.shape[i];
  }
  
  public RnSpc(int[] shape) {
    this.shape = shape;
    n = 1;
    for (int i = 0; i < shape.length; i++ ) {
      n *= shape[i]; }}
  
  /** Constructs a one-dimensional RnSpc (i.e. a space of column vectors) */
  public RnSpc(int n1) {
    this.shape    = new int[1];
    this.shape[0] = n1;
    n = n1; }
  
  /** Constructs an <TT>RnSpc</TT> which is a subspace of superSpace.<BR>
   *  Any dimension, i, in which <TT>subShape[i] == superSpace.shape[i]</TT>
   *  is included as a dimension in the new <TT>RnSpc</TT>.<BR>
   *  All other dimensions are collapsed.
   */
  /*  protected RnSpc(RnSpc superSpace, int[] subShape) {
      n = 0;
      for (int i = 0; i < superSpace.n; i++)
      if (subShape[i] == superSpace.shape[i]) n++;
      shape = new int[n];
      j = 0;
      for (int i = 0; i < superSpace.n; i++)
      if (subShape[i] == superSpace.shape[i])
      shape[j++] = superSpace.shape[i]; }
      */

  public JamObject copy()     { return new RnSpc(this); }

  public boolean   equals(Object obj2) {
    if (this == obj2) return true;
    if (! (obj2 instanceof RnSpc)) return false;
    RnSpc spc2 = (RnSpc) obj2;
    if (this.n != spc2.n) return false;
    for (int i = 0; i < this.shape.length; i++)
      if (this.shape[i] != spc2.shape[i]) return false;
    return true;
  }
  
  public int[] getShape() { return shape; }
  /**
   * @return The total number of components in a vector member of this space.<BR>(That is the basis-dimension of this space).
   */
  public int   size    () { return n    ; }
  
  /**
   * @return The (1D) integer index for an array which corresponds to index
   */
  public int   vecIndex(int[] index) {
    int ind = 0;
    for (int dim = Math.min(shape.length, index.length ) - 1;
	 dim > 0; dim-- ) {
      ind = (index[dim] + ind) * shape[dim-1];
    }
    return index[0] + ind;
  }
  
  public Vector newMember() {
    return (Vector) new RnVec(this);
  }
  
  public boolean hasMember(Vector v) {
    return v.getSpace().equals(this);
  }
  
  public String toString() {
    StringBuffer shp = new StringBuffer();
    for (int i = 0; i < shape.length; i++) {
      shp.append(shape[i] + " "); }
    return ("RnSpc:" + "\n" +
	    "Shape :" + shp); }
}
