package  juice.example;

import   jam.vector.*;
import   jam.JamObject;

/**
 * RnVec is a vector in a space of type RnSpc
 * <BR><!--#include file="./source.inc"-->
 * 
 * @see         RnSpc
 * @see         RnOp
 * @author      Joel Schroeder
 */
public class RnVec implements Vector {
  protected RnSpc    space;
  protected float[]  data;

  protected RnVec() {}
  
  public RnVec(RnSpc space) {
    this.space = space;
    this.data  = new float[space.size()];
  }
  
  public RnVec(int[] shape) {
    this(new RnSpc(shape));
  }
  
  public RnVec(int n1) {
    this(new RnSpc(n1));
  }

  /** copy constructor */
  protected RnVec(RnVec src) {
    this.space = src.space;
    this.data  = new float[src.data.length];
    for (int i = 0; i < this.data.length; i++)
      this.data[i] = src.data[i];
  }

  public JamObject copy()     { return new RnVec(this); }

  public boolean   equals(Object obj2) {
    if (this == obj2) return true;
    if (! (obj2 instanceof RnVec)) return false;
    RnVec vec2 = (RnVec) obj2;
    if (this.space != vec2.space) return false;
    for (int i = 0; i < this.data.length; i++)
      if (this.data[i] != vec2.data[i]) return false;
    return true;
  }
  
  public float[] getData() { return data; }
  
  public Space        getSpace() { return space; }
  
  public void         neg     () {
    for (int i = 0; i < data.length; i++ )
      this.data[i] = - this.data[i]; }
  public void         neg     (Vector vec2) {
    for (int i = 0; i < data.length; i++ ) {
      this.data[i] = - ((RnVec) vec2).data[i]; }}
  public void         add     (Vector vec2) {
    for (int i = 0; i < data.length; i++ ) {
      this.data[i] += ((RnVec) vec2).data[i]; }}
  public void         add     (Vector vec1,
			       Vector vec2) {
    for (int i = 0; i < data.length; i++ ) {
      this.data[i] = ((RnVec) vec1).data[i] + ((RnVec) vec2).data[i];
    }}
  public void         addScale(float a,
			       Vector vec2) {
    for (int i = 0; i < data.length; i++ )
      this.data[i] += a*((RnVec) vec2).data[i];
  }
  public void         addScale(float a,
			       Vector vec1,
			       Vector vec2) {
    for (int i = 0; i < data.length; i++ )
      this.data[i]  = a*((RnVec) vec2).data[i] + ((RnVec) vec1).data[i];
  }
  public void         sub     (Vector vec2) {
    for (int i = 0; i < data.length; i++ ) {
      this.data[i] -= ((RnVec) vec2).data[i]; }}
  public void         sub     (Vector vec1,
			       Vector vec2) {
    for (int i = 0; i < data.length; i++ ) {
      this.data[i] = ((RnVec) vec1).data[i] - ((RnVec) vec2).data[i];
    }}
  public void         scale   (float        a    ) {
    for (int i = 0; i < data.length; i++ ) {
      this.data[i] *= a; }}
  public void         scale   (float        a    ,
			       Vector vec ) {
    for (int i = 0; i < data.length; i++ ) {
      this.data[i] = a * ((RnVec) vec).data[i]; }}

  public void         copyFrom(Vector src) {
    for (int i = 0; i < data.length; i++ ) {
      this.data[i] = ((RnVec) src).data[i]; }}
  public void         zero    () { fill(0F); }
  public float 	      dot     (Vector vec2) {
    float product = 0F;
    for (int i = 0; i < data.length; i++ ) {
      product += this.data[i] * ((RnVec) vec2).data[i]; }
    return product; }
  public float 	      norm2   () {
    float product = 0F;
    for (int i = 0; i < data.length; i++ ) {
      product += this.data[i] * this.data[i]; }
    return product; }
  public float 	      norm    () { return (float) Math.sqrt(norm2()); }
  public float 	      getVal  (int[] index) {
    return data[ space.vecIndex(index)]; }
  public void 	      setVal  (int[] index, float val) {
    data[ space.vecIndex(index)] = val;  }
  public void  	      fill    (float a) {
    for (int i = 0; i < data.length; i++ ) {
      this.data[i] = a; }}
  public void  	      rand    () {
    for (int i = 0; i < data.length; i++ ) {
      this.data[i] = 2*((float) Math.random()) - 1F; }}
  
  public String toString() {
    StringBuffer values = new StringBuffer();
    for (int i = 0; i < space.size(); i++) {
      values.append(data[i] + " "); }
    return ("RnVec:" + "\n" +
	    "Space :" + space.toString() + "\n" +
	    "Values:" + values); }
}
