package juice.operator;

import    jam.operator.*;
import    jam.vector.*;
import  juice.vector.*;

/**
 * <BR>
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @author      Joel Schroeder
 */
public class BlockLinOpAdj extends BlockOp
implements BlockLinearOperatorAdjoint {
  protected BlockLinOpAdj adj      = null;
  protected boolean       setOpAdj = true;

  public BlockLinOpAdj(int nRows, int nCols) { super(nRows, nCols); }

  /**
   * This constructor assumes that if the domain or range is a
   * <TT>ProductSpace</TT> space, then the number of columns or
   * rows, respectively, equals the number of sub-spaces in the
   * product space.<BR>
   * Therefore, if the domain/range is a <TT>ProductSpace</TT>,
   * but you want the <TT>BlockOp</TT> to have only one column/row,
   * respectively, you SHOULD NOT USE THIS CONSTRUCTOR, since it
   * will make the number of columns/rows equal to the number of
   * sub-spaces in the product-space.
   */
  public BlockLinOpAdj(Space domain, Space range) { super(domain, range); }

  public BlockLinOpAdj(Space domain, int nCols, Space range, int nRows) {
    super(domain, nCols, range, nRows);
  }

  void    setOp(int index, Operator op) {
    super.setOp(index, op);
    if (adj != null) {
      if (setOpAdj == true) {
	adj.setOpAdj = false;
	adj.setOp(adjIndex(index), op);
      }
      else
	setOpAdj = true;
    }
  }

  public void createAdjoint() {
    adj = new BlockLinOpAdj(nCols(), nRows());
    for   (int i = 0; i < nRows; i++) {
      for (int j = 0; j < nCols; j++) {
	Operator opIJ = getOp(i, j);
	if (opIJ != null) adj.setOp(j, i, ((hasAdjoint) opIJ).adjoint());
      }}
    adj.adj = this;
  }

  public LinearOperator getGradientAt  (Vector x) { return adjoint(); }
  public hasAdjoint     getDerivativeAt(Vector x) { return this; }
  public hasAdjoint     adjoint() {
    if (adj == null) createAdjoint();
    return adj;
  }

  final private int adjIndex(int index) {
    int i = index / nCols;
    return (index - i*nCols) * nRows + i;
  }

  public String toString() {
    return("BlockLinOpAdj, " + nRows + " by " + nCols + ":\n" +
	   "Operators:\n" + opList);
  }

  public jam.JamObject copy() {
    System.err.println("BlockLinOpAdj.copy() not implemented.\n");
    return null;
  }
}
