package juice.operator;

import    jam.JamObject;
import    jam.operator.*;
import    jam.vector.*;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @author      Joel Schroeder
 */
public class CompoundLinOpAdj extends CompoundOpGrad
implements CompoundLinearOperatorAdjoint {
  protected  CompoundLinOpAdj adj = null;

  /** copy constructor */
  protected  CompoundLinOpAdj(CompoundLinOpAdj src) { super(src); }

  public     CompoundLinOpAdj()      {}
  protected  CompoundLinOpAdj(int n) { super(n); }

  public     CompoundLinOpAdj(hasAdjoint first) {
    this();
    addOpLast(first );
  }

  public     CompoundLinOpAdj(hasAdjoint first, hasAdjoint second) {
    this(first );
    addOpLast(second);
  }

  public     CompoundLinOpAdj(hasAdjoint first, hasAdjoint second,
			      hasAdjoint third) {
    this(first, second);
    addOpLast(third );
  }

  public JamObject copy  () { return new CompoundLinOpAdj(this); }

  public boolean  equals(Object obj2) {
    if (! (obj2 instanceof CompoundLinOpAdj)) return false;
    return super.equals(obj2);
  }

  protected boolean consistentType(Operator op) {
    return (op instanceof hasAdjoint);
  }

  public void     addOpFirst(Operator first) {
    if (! consistentType(first)) {
      System.err.println("Error in CompoundLinOpAdj.addOpFirst(): " +
			 "inconsistent operator type for argument first.");
      return;
    }
    super.addOpFirst(first);
  }
  public void     addOpLast (Operator last ) {
    if (! consistentType(last)) {
      System.err.println("Error in CompoundLinOpAdj.addOpLast(): " +
			 "inconsistent operator type for argument last.");
      return;
    }
    super.addOpLast(last);
  }

  public void     setOp(int i, Operator op) {
    if (! consistentType(op)) {
      System.err.println("Error in CompoundLinOpAdj.setOp(): " +
			 "inconsistent operator type for argument op.");
      return;
    }
    super.setOp(i, op);
  }

  public hasAdjoint adjoint() {
    if (adj == null)
      createAdjoint();
    return adj;
  }

  public hasAdjoint getDerivativeAt(Vector x) { return this; }

  void createAdjoint() {
    int n = this.size();
    adj = new CompoundLinOpAdj(n);
    for (int i = 0; i < n; i++) {
      hasAdjoint currOp = (hasAdjoint) this.getOp(i);
      adj.setOp(n - 1 - i, (currOp == null) ? null:currOp.adjoint());
    }
    adj.adj = this;
  }

  public String toString() {
    return ("CompoundLinOpAdj of " + size() + " operators:\n\n" + opList);
  }
}
