package juice.operator;

import    jam.JamObject;
import    jam.operator.*;
import    jam.vector.*;
import   java.util.Enumeration;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @author      Joel Schroeder
 */
public class CompoundOp implements CompoundOperator {
  protected OpList opList;

  /** copy constructor */
  protected  CompoundOp(CompoundOp src) {
    this.opList = new OpList(src.opList);
  }

  public     CompoundOp()      { opList = new OpList(); }
  protected  CompoundOp(int n) { opList = new OpList(n); }

  public     CompoundOp(Operator first) {
    this();
    addOpLast(first );
  }

  public     CompoundOp(Operator first, Operator second) {
    this(first );
    addOpLast(second);
  }

  public     CompoundOp(Operator first, Operator second, Operator third) {
    this(first, second);
    addOpLast(third );
  }

  public JamObject copy  () { return new CompoundOp(this); }

  public boolean   equals(Object obj2) {
    if (this == obj2) return true;
    if (! (obj2 instanceof CompoundOp)) return false;

    CompoundOp sop2 = (CompoundOp) obj2;
    return (this.opList.equals(sop2.opList));
  }

  /**
   * consistentSpaces() does space checking on the domains and ranges.
   * @param i any integer in [-1,n]
   */
  protected boolean  consistentSpaces(Operator op, int i) {
    int n = this.size();
    if ((i > n) || (i < -1)) return false;
    Operator prev = null, next = null;
    if (i > 0)   prev = getOp(i-1);
    if (i < n-1) next = getOp(i+1);
    if ((prev != null) && !(prev.getRange() .equals(op.getDomain())))
      return false;
    if ((next != null) && !(next.getDomain().equals(op.getRange ())))
      return false;
    return true;
  }

  public int      size() { return opList.size(); }

  public void     addOpFirst(Operator first) {
    if ((size() != 0) && (! consistentSpaces(first,-1))) {
      System.err.println("Error in addOpFirst(): " +
			 "inconsistent range of argument first.");
      return;
    }
    opList.addOpFirst(first);
  }
  public void     addOpLast (Operator last ) {
    int n = size();
    if ((n != 0) && (! consistentSpaces(last, n))) {
      System.err.println("Error in addOpLast(): " +
			 "inconsistent domain of argument last.");
      return;
    }
    opList.addOpLast(last);
  }

  public Operator getOp(int i) { return opList.getOp(i); }
  public void     setOp(int i, Operator op) {
    if ((size() != 0) && (! consistentSpaces(op, i))) {
      System.err.println("Error in setOp(): " +
			 "inconsistent domain or range of argument op.");
      return;
    }
    opList.setOp(i, op);
  }

  public Enumeration  elements () { return opList.elements(); }

  public    Space     getDomain() {
    Operator op = (Operator) opList.ops.firstElement();
    return (op == null) ? null:op.getDomain();
  }
  public    Space     getRange () {
    Operator op = (Operator) opList.ops.lastElement();
    return (op == null) ? null:op.getRange();
  }

  protected Vector    image(Vector d, int numOps) {
    Vector next = d;
    for (Enumeration e = elements(); numOps > 0; numOps--) {
      Operator nextOp = (Operator) e.nextElement();
      Vector last = next;
      next = nextOp.image(last);
    }
    return next;
  }

  public Vector       image(Vector d) {
    return image(d, opList.size());
  }
    
  public void         image(Vector d, Vector r) {
    ((Operator) opList.ops.lastElement())
      .image(image(d, opList.size() - 1), r);
  }

  public void      addImage(Vector d, Vector r) {
    ((Operator) opList.ops.lastElement())
      .addImage(image(d, opList.size() - 1), r);
  }

  public void      residual(Vector d, Vector b, Vector r) {
    ((Operator) opList.ops.lastElement())
      .residual(image(d, opList.size() - 1), b, r);
  }

  public String toString() {
    return ("CompoundOp of " + size() + " operators:\n\n" + opList);
  }
}
