package juice.operator;

import    jam.JamObject;
import    jam.operator.*;
import    jam.vector.*;
import   java.util.Enumeration;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 * Do we want to actually extend CompoundOp or simply implement
 * CompoundOperator?
 *
 * @author      Joel Schroeder
 */
public class CompoundOpGrad extends CompoundOp
implements hasGradient {

  /** copy constructor */
  public     CompoundOpGrad(CompoundOpGrad src) {
    super(src);
  }

  public     CompoundOpGrad()      {}
  protected  CompoundOpGrad(int n) { super(n); }

  public     CompoundOpGrad(hasGradient first) {
    this();
    addOpLast(first );
  }

  public     CompoundOpGrad(hasGradient first, hasGradient second) {
    this(first );
    addOpLast(second);
  }

  public     CompoundOpGrad(hasGradient first, hasGradient second,
			    hasGradient third) {
    this(first, second);
    addOpLast(third );
  }

  public JamObject copy  () { return new CompoundOpGrad(this); }

  public boolean   equals(Object obj2) {
    if (! (obj2 instanceof CompoundOpGrad)) return false;
    return super.equals(obj2);
  }

  protected boolean consistentType(Operator op) {
    return (op instanceof hasGradient);
  }

  public void     addOpFirst(Operator first) {
    if (! consistentType(first)) {
      System.err.println("Error in addOpFirst(): " +
			 "inconsistent operator type for argument first.");
      return;
    }
    super.addOpFirst(first);
  }
  public void     addOpLast (Operator last ) {
    if (! consistentType(last)) {
      System.err.println("Error in addOpLast(): " +
			 "inconsistent operator type for argument last.");
      return;
    }
    super.addOpLast(last);
  }

  public void     setOp(int i, Operator op) {
    if (! consistentType(op)) {
      System.err.println("Error in setOp(): " +
			 "inconsistent operator type for argument op.");
      return;
    }
    super.setOp(i, op);
  }

  public LinearOperator     getGradientAt(Vector x) {
    int n = this.size();
    CompoundLinOp grad = new CompoundLinOp(n);

    hasGradient currentOp    = (hasGradient) this.getOp(0);
    grad.setOp(n - 1, currentOp.getGradientAt(x));
    Vector   currentImage = x;
    for (int i = 1; i < n; i++) {
      currentImage = currentOp.image(currentImage);
      currentOp    = (hasGradient) this.getOp(i);
      grad.setOp(n - 1 - i, currentOp.getGradientAt(currentImage));
    }
    return grad;
  }

  public String toString() {
    return ("CompoundOpGrad of " + size() + " operators:\n" + opList);
  }
}
