package juice.operator;

import    jam.operator.*;
import    jam.vector.*;
import  juice.vector.*;
import    jam.JamObject;

/**
 * This operator implements NormOp.image(x) = x.dot(x) 
 *
 * <BR><!--#include file="./source.inc"--><BR>
 * @author      Matt Schwab
 * @version     modified by Joel in moving to jam
 */
public class NormOp implements hasGradient {
  protected Space       domain;

  /*
  public NormOp(Space domain, RealSpc range) {
    //public     NormOp(Space domain, Space range) {
    // check that your range is a RealSpc
    // maybe we should only allow RealSpc in call
    this.domain     = domain;
    this.derivative = new IdentityOp(this.domain, this.domain);
  }
  */

  public     NormOp(Space domain) { this.domain = domain; }

  protected  NormOp(NormOp src)   { this.domain = src.domain; }

  public Space 	      getDomain() { return domain; }
  public Space 	      getRange () { return RealSpc.getSpace(); }

  public LinearOperator  getGradientAt(Vector x) { 
    Vector twiceX = (Vector) x.copy();
    twiceX.scale(2F);
    return new OpVec(twiceX);
  }

  public Vector  image(Vector d) {
    return new RealVec(d.norm2());
  }

  public void    image(Vector d, Vector r) {
    RealVec rr = (RealVec) r;
    rr.setValue(d.norm2());
  }

  public void addImage(Vector d, Vector r) {
    RealVec rr = (RealVec) r;
    rr.setValue(d.norm2() + rr.getValue());
  }

  public void residual(Vector d, Vector b, Vector r) {
    RealVec rr = (RealVec) r;
    RealVec bb = (RealVec) b;

    rr.setValue(d.norm2() - bb.getValue());
  }

  public boolean equals(Object obj2) {
    if (this == obj2) return true;
    if (! (obj2 instanceof NormOp)) return false;
    return (this.getDomain().equals(((Operator) obj2).getDomain()));
  }

  public JamObject copy()         { return new NormOp(this); }

  public String          toString () {
    return ("NormOp:"    + "\n" +
	    "Domain:" + domain + "\n" +
	    "Range : RealSpc"         );
  }
}



