package juice.operator;

import    jam.JamObject;
import    jam.vector.*;
import    jam.operator.*;
import  juice.vector.RealVec;

/**
 * <BR><!--#include file="./source.inc"-->
 *
 * @author      Joel Schroeder
 */
public class OpVec implements OperatorVector {
  public    Vector  vec;
  protected OpVec   adj;
  protected boolean isFwd;

  /** copy constructor */
  public     OpVec(OperatorVector src) {
    this.vec = (Vector) src.getVector().copy();
    isFwd    = (src.getDomain() instanceof RealSpace);
    adj      = null;
  }
  public     OpVec(Vector vec)         {
    this.vec = vec;
    isFwd    = true;
    adj      = null;
  }

  public Vector    getVector()    { return vec; }
  public JamObject copy()         { return new OpVec(this); }

  public boolean   equals(Object obj) {
    if (this == obj) return true;
    if (! (obj instanceof OperatorVector)) return false;
    return (vec.equals( ((OperatorVector) obj).getVector()));
  }

  public  Space getDomain() {
    return (isFwd ? getInputSpace() :getOutputSpace());
  }
  public  Space getRange () {
    return (isFwd ? getOutputSpace():getInputSpace() );
  }

  private Space getInputSpace () { return juice.vector.RealSpc.getSpace(); }
  private Space getOutputSpace() { return vec.getSpace(); }

  public Vector  image(Vector d) {
    Vector r = getRange().newMember();
    image(d, r);
    return r;
  }

  public void    image(Vector d, Vector r) {
    if (isFwd)
      r .scale(((RealVector) d).getValue(), vec);
    else
      ((RealVector) r).setValue(vec.dot(d));
  }
  
  public void addImage(Vector d, Vector r) {
    if (isFwd)
      r .addScale(((RealVector) d).getValue(), vec);
    else {
      RealVector rr = (RealVector) r;
      rr.setValue(rr.getValue() + vec.dot(d));
    }
  }

  public void residual(Vector d, Vector b, Vector r) {
    r.neg(b);
    addImage(d, r);
  }

  public LinearOperator getGradientAt  (Vector x) { return adjoint(); }
  public hasAdjoint     getDerivativeAt(Vector x) { return this;      }

  protected void createAdjoint() {
    adj = new OpVec(vec);
    adj.isFwd = !this.isFwd;
    adj.adj   =  this;
  }

  public hasAdjoint adjoint() {
    if (adj == null) createAdjoint();
    return adj;
  }

  public String toString() { return("OpVec:\n" + vec); }
}
