package juice.operator;

import    jam.operator.*;
import    jam.vector.*;

/**
 * A RankOneOperator is defined by two vectors, u and v.<BR>
 * The operator is just the "matrix" uv'<BR>
 * Thus the operator acting on x gives (v'x)u, i.e. u scaled by a dot product
 * <BR><!--#include file="./source.inc"-->
 */
public class RankOneOperator implements hasAdjoint {
  protected Vector 	    u, v;
  protected RankOneOperator adj = null;

  /**
   * Creates the (rank one) "matrix" operator uv'
   */
  public RankOneOperator(Vector u, Vector v) {
    this.u = u;
    this.v = v;
  }

  public Space  getDomain() { return v.getSpace(); }
  public Space  getRange () { return u.getSpace(); }

  public Vector  image(Vector d) {
    Vector r = (Vector) u.copy();
    r.scale(v.dot(u));
    return r;
  }
  public void    image(Vector d, Vector r) { r.   scale(v.dot(d), u); }
  public void addImage(Vector d, Vector r) { r.addScale(v.dot(d), u); }
  public void residual(Vector d, Vector b, Vector r) {
    r.neg(b);
    addImage(d, r);
  }

  void createAdjoint() {
    adj = new RankOneOperator(v, u);
    adj.adj = this;
  }

  public hasAdjoint     adjoint() {
    if (adj == null) createAdjoint();
    return adj;
  }

  public hasAdjoint     getDerivativeAt(Vector x) {
    return this;
  }
  public LinearOperator getGradientAt  (Vector x) {
    return adjoint();
  }

  public boolean equals(Object obj2) {
    if (this == obj2) return true;
    if (! (obj2 instanceof RankOneOperator)) return false;
    RankOneOperator roo2 = (RankOneOperator) obj2;
    return (this.u.equals(roo2.u) &&
	    this.v.equals(roo2.v));
  }

  public jam.JamObject copy() {
    return new RankOneOperator((Vector) u.copy(), (Vector) v.copy());
  }
}
