package juice.operator;

import    jam.operator.*;
import    jam.vector.*;
import  juice.vector.*;
import    jam.JamObject;

/**
 * A <TT>ResidualOp</TT>, R, is defined by an <TT>Operator</TT>, A, and
 * a <TT>Vector</TT> in the range of A, call it b, as:<BR>
 * <BR>
 * R(x) = A(x) - b
 *
 * <BR><!--#include file="./source.inc"--><BR>
 * @author      Matt Schwab
 * @version     7 Mar 1997
 */
public class ResidualOp implements Operator {
  protected Operator A;
  protected Vector   b;

  public     ResidualOp(Operator A, Vector b) {
    if (! b.getSpace().equals(A.getRange())) {
      System.err.println("Operator A range differs from space of b");
    }
    this.A = A;
    this.b = b;
  }

  protected ResidualOp(ResidualOp src) {
    this.A = (Operator) src.A.copy();
    this.b = (Vector)   src.b.copy();
  }

  public Space getDomain() { return A.getDomain(); }
  public Space getRange () { return A.getRange();  }

  public Vector  image(Vector d) {
    Vector r = getRange().newMember();
    A.residual(d, b, r);
    return r; 
  }

  public void    image(Vector d, Vector r) {
    A.residual(d, b, r);
  }

  public void addImage(Vector d, Vector r) {
    A.addImage(d, r); 
    r.sub(b);
  }

  public void residual(Vector d, Vector b, Vector r) {
    this.image(d, r);
    r.sub(b);
  }

  /** 
   * Is there any reason to check if two operators are equal?
   */
  public boolean equals(Object obj2) {
    if (this == obj2) return true;
    if (! (obj2 instanceof ResidualOp)) return false;
    ResidualOp rop2 = (ResidualOp) obj2;
    return (this.A.equals(rop2.A) &&
	    this.b.equals(rop2.b));
  }

  public JamObject copy() { return new ResidualOp(this); }

  public String          toString () {
    return ("ResidualOp: A(x) - b" + "\n" +
	    "  operator: "        + "\n" + A + "\n" +
	    "  vector  : "        + "\n" + b + "\n" ); 
  }
}
