package juice.operator;

import    jam.JamObject;
import    jam.operator.*;
import    jam.vector.*;

/**
 * A <TT>ScaledOp</TT> is an implementation of <TT>ScaledOperator</TT>.
 * <BR><!--#include file="./source.inc"--><BR>
 * 
 * @author      Joel Schroeder
 */
public class ScaledOp implements ScaledOperator {
  protected float    sclr;
  protected Operator op;

  public ScaledOp(Operator op, float sclr) {
    this.op = op;
    setScale(sclr);
  }

  /** copy constructor */
  protected ScaledOp(ScaledOp src) {
    src.op   = (Operator) this.op.copy();
    src.sclr = this.sclr;
  }

  public JamObject copy() { return new ScaledOp(this); }

  public boolean equals(Object obj2) {
    if (this == obj2) return true;
    if (! (obj2 instanceof ScaledOp)) return false;
    ScaledOp sop2 = (ScaledOp) obj2;
    return (this.sclr == sop2.sclr &&
	    this.op.equals(sop2.op));
  }

  public Operator  getOp    ()           { return op; }
  public float     getScale ()           { return sclr; }
  public void      setScale (float sclr) { this.sclr = sclr; }

  public Space 	   getDomain() { return op.getDomain(); } 
  public Space 	   getRange () { return op.getRange (); }

  public Vector    image    (Vector d) {
    Vector r = op.image(d);
    if (sclr != 1F) r.scale(sclr);
    return r;
  }
  public void      image    (Vector d, Vector r) {
    op.image(d, r);
    if (sclr != 1F) r.scale(sclr);
  }
  public void   addImage    (Vector d, Vector r) {
    if (sclr == 1F) op.addImage(d, r);
    else {
      Vector im = op.image(d);
      r.addScale(sclr, im);
    }
  }
  public void   residual    (Vector d, Vector b, Vector r) {
    r.neg(b);
    addImage(d, r);
  }
}
