package juice.operator;

import    jam.JamObject;
import    jam.operator.*;
import    jam.vector.*;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @author      Joel Schroeder
 */
public class SummedLinOpAdj extends SummedOp implements hasAdjoint {
  protected SummedLinOpAdj adj = null;

  /** copy constructor */
  protected  SummedLinOpAdj(SummedLinOpAdj src) {
    this.opList = new OpList(src.opList);
  }

  public     SummedLinOpAdj()      { opList = new OpList(); }

  protected  SummedLinOpAdj(int n) { opList = new OpList(n); }

  public     SummedLinOpAdj(hasAdjoint first) {
    this ();
    addOp(first );
  }

  public     SummedLinOpAdj(hasAdjoint first, hasAdjoint second) {
    this (first );
    addOp(second);
  }

  public     SummedLinOpAdj(hasAdjoint first, hasAdjoint second,
			    hasAdjoint third) {
    this (first, second);
    addOp(third );
  }

  public JamObject copy  () { return new SummedLinOpAdj(this); }

  /**
   * equals() is true only if the order of sub-operators is the same (even
   * though, mathematically, addition is commutative).  That is, if:<BR>
   * X = (A + B) but <BR>
   * Y = (B + A)     <BR>
   * then X.equals(Y) returns false
   */
  public boolean  equals(Object obj2) {
    if (! (obj2 instanceof SummedLinOpAdj)) return false;
    return super.equals(obj2);
  }

  protected boolean consistentType(Operator op) {
    return (op instanceof hasAdjoint);
  }

  public void     addOpFirst(Operator first) {
    if (! consistentType(first)) {
      System.err.println("Error in SummedLinOpAdj.addOpFirst(): " +
			 "inconsistent operator type of argument op.");
      return;
    }
    super.addOpFirst(first);
  }
  public void     addOpLast (Operator last ) {
    if (! consistentType(last)) {
      System.err.println("Error in SummedLinOpAdj.addOpLast(): " +
			 "inconsistent operator type of argument op.");
      return;
    }
    super.addOpLast(last);
  }

  public void     setOp(int i, Operator op) {
    if (! consistentType(op)) {
      System.err.println("Error in SummedLinOpAdj.setOp(): " +
			 "inconsistent operator type of argument op.");
      return;
    }
    super.setOp(i, op);
  }

  public hasAdjoint adjoint() {
    if (adj == null) createAdjoint();
    return adj;
  }

  public hasAdjoint     getDerivativeAt(Vector x) { return this     ; }
  public LinearOperator getGradientAt  (Vector x) { return adjoint(); }

  void createAdjoint() {
    int n = this.size();
    adj = new SummedLinOpAdj(n);
    for (int i = 0; i < n; i++) {
      hasAdjoint currOp = (hasAdjoint) this.getOp(i);
      adj.setOp(i, (currOp == null) ? null:currOp.adjoint());
    }
    adj.adj = this;
  }
}
