package juice.operator;

import    jam.JamObject;
import    jam.operator.*;
import    jam.vector.*;
import   java.util.Enumeration;

/**
 * Implementation of <TT>SummedOperator</TT><BR>
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @author      Joel Schroeder
 */
public class SummedOp implements SummedOperator {
  OpList opList;

  /** copy constructor */
  protected  SummedOp(SummedOp src) {
    this.opList = new OpList(src.opList);
  }

  public     SummedOp() { opList = new OpList(); }

  public     SummedOp(Operator first) {
    this ();
    addOp(first );
  }

  public     SummedOp(Operator first, Operator second) {
    this (first );
    addOp(second);
  }

  public     SummedOp(Operator first, Operator second, Operator third) {
    this (first, second);
    addOp(third );
  }

  public JamObject copy  () { return new SummedOp(this); }

  /**
   * equals() is true only if the order of sub-operators is the same (even
   * though, mathematically, addition is commutative).  That is, if:<BR>
   * X = (A + B) but <BR>
   * Y = (B + A)     <BR>
   * then X.equals(Y) returns false
   */
  public boolean   equals(Object obj2) {
    if (this == obj2) return true;
    if (! (obj2 instanceof SummedOp)) return false;

    SummedOp sop2 = (SummedOp) obj2;
    return (this.opList.equals(sop2.opList));
  }

  /**
   * true if op's domain and range equal those of "this"
   */
  protected boolean  consistentSpaces(Operator op) {
    return (op.getDomain().equals(this.getDomain()) &&
	    op.getRange ().equals(this.getRange ()));
  }

  public int      size() { return opList.size(); }

  public void     addOpFirst(Operator first) {
    if ((size() != 0) && (! consistentSpaces(first))) {
      System.err.println("Error in addOpFirst(): " +
			 "inconsistent domain or range of argument first.");
      return;
    }
    opList.addOpFirst(first);
  }
  public void     addOpLast (Operator last ) {
    if ((size() != 0) && (! consistentSpaces(last))) {
      System.err.println("Error in addOpLast(): " +
			 "inconsistent domain or range of argument last.");
      return;
    }
    opList.addOpLast(last);
  }

  public void     addOp(Operator op) { addOpLast(op); }

  public Operator getOp(int i) { return opList.getOp(i); }
  public void     setOp(int i, Operator op) {
    if ((size() != 0) && (! consistentSpaces(op))) {
      System.err.println("Error in setOp(): " +
			 "inconsistent domain or range of argument op.");
      return;
    }
    opList.setOp(i, op);
  }

  public Enumeration  elements() { return opList.elements(); }

  public    Space     getDomain	 ()   {
    Operator op = (Operator) opList.ops.firstElement();
    return (op == null) ? null:op.getDomain();
  }
  public    Space     getRange 	 ()   {
    Operator op = (Operator) opList.ops.firstElement();
    return (op == null) ? null:op.getRange();
  }

  protected void      addImage(Vector d,
			       Vector r, Enumeration e) {
    while (e.hasMoreElements())
      ((Operator) e.nextElement()).addImage(d, r);
  }

  public    Vector    image   (Vector d) {
    Enumeration e = opList.elements();
    Vector r = ((Operator) e.nextElement()).image(d);
    addImage(d, r, e);
    return r;
  }

  public    void      image   (Vector d,
			       Vector r) {
    Enumeration e = opList.elements();
    ((Operator) e.nextElement()).image(d, r);
    addImage(d, r, e);
  }

  public    void      addImage(Vector d,
			       Vector r) {
    addImage(d, r, opList.elements());
  }

  public    void      residual(Vector d,
			       Vector b,
			       Vector r) {
    r.neg(b);
    addImage(d, r);
  }
}
