package juice.operator;

import  juice.vector.*;
import    jam.vector.*;
import    jam.operator.*;
import    jam.JamObject;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @author      Joel Schroeder
 */
final public class ZeroBiLinOp implements BiLinearOperatorAdjoint {
  ProductSpace domain;
  Space        range;

  public     ZeroBiLinOp(ProductSpace domain, Space range) {
    this.domain = domain;
    this.range  = range;
  }

  public     ZeroBiLinOp(Space domain1, Space domain2, Space range) {
    this(new ProductSpc(domain1, domain2), range);
  }

  ZeroBiLinOp(ZeroBiLinOp src) {
    this((ProductSpace) src.domain.copy(), (Space) src.range.copy());
  }

  public Space getDomain1() { return domain.getSubSpace(0); }
  public Space getDomain2() { return domain.getSubSpace(1); }
  public Space getDomain () { return domain; }
  public Space getRange  () { return range ; }

  public Vector      imageBinary(Vector d1, Vector d2          ) {
    Vector r = range.newMember();
    r.zero();
    return r;
  }
  public void        imageBinary(Vector d1, Vector d2, Vector r) {
    r.zero();
  }
  public void     addImageBinary(Vector d1, Vector d2, Vector r) {}
  public void     residualBinary(Vector d1, Vector d2,
 			         Vector b , Vector r) {
    r.neg(b);
  }

  public Vector       image   (Vector d) {
    Vector r = range.newMember(); r.zero();
    return r;
  }
  public void         image   (Vector d, Vector r) {
    r.zero();
  }
  public void         addImage(Vector d, Vector r) {}
  public void         residual(Vector d, Vector b, Vector r) {
    r.neg(b);
  }

  public Operator getOp1(Vector d1) {
    return new ZeroOp(domain.getSubSpace(1), range);
  }
  public Operator getOp2(Vector d2) {
    return new ZeroOp(domain.getSubSpace(0), range);
  }
  public hasAdjoint getPartialAdjoint(Vector r) {
    return new ZeroOp(domain.getSubSpace(1),
		      domain.getSubSpace(0));
  }

  public JamObject copy() { return new ZeroBiLinOp(this); }

  public boolean equals(Object obj2) {
    if (this == obj2) return true;
    if (! (obj2 instanceof ZeroBiLinOp)) return false;
    return (this.domain.equals(((ZeroBiLinOp) obj2).domain) &&
	    this.range .equals(((ZeroBiLinOp) obj2).range ));
  }
}
