package juice.operator;

import  jam.vector.*;
import  jam.operator.*;
import  jam.JamObject;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @author      Joel Schroeder
 */
public class ZeroOp implements ZeroOperator {
  protected  Space  domain;
  protected  Space  range ;
  protected  ZeroOp adj   = null;

  public     ZeroOp(Space domain, Space range) {
    this.domain = domain;
    this.range  = range ;
  }

  public     ZeroOp(Space domainAndRange) {
    this(domainAndRange, domainAndRange);
  }

  public boolean   equals(Object obj2) {
    if (this == obj2) return true;
    if (! (obj2 instanceof ZeroOp)) return false;
    ZeroOp zop2 = (ZeroOp) obj2;
    return (this.domain.equals(zop2.domain) &&
	    this.range .equals(zop2.range ));
  }

  public JamObject       copy  () { return new ZeroOp(domain, range); }

  public Space 	        getDomain() { return domain; }
  public Space 	        getRange () { return range ; }

  public LinearOperator getGradientAt  (Vector x) { return adjoint(); }
  public hasAdjoint     getDerivativeAt(Vector x) { return this;      }

  public BiLinearOperatorAdjoint getHessianAt(Vector x) {
    return new ZeroBiLinOp(domain, domain, range);
  }

  public Vector          image(Vector d) {
    Vector r = (Vector) range.newMember();
    r.zero();
    return r;
  }
    
  public void            image(Vector d,
			       Vector r) {
    r.zero();
  }

  public void         addImage(Vector d,
			       Vector r) {
  }
  
  public void         residual(Vector d,
			       Vector b,
			       Vector r) {
    r.neg(b);
  }

  protected void createAdjoint() {
    adj = new ZeroOp(range, domain);
    adj.adj = this;
  }

  public hasAdjoint adjoint() {
    if (adj == null) createAdjoint();
    return adj;
  }
  public String       toString() {
    return ("ZeroOp:"          + "\n" +
	    "Domain:" + domain + "\n" +
	    "Range :" + range          );
  }
}
