package juice.solver;

import  jam.solver.*;
import  jam.vector.*;
import  jam.operator.*;
import  juice.vector.*;
/**
 * Conjugate Gradient Solver.<BR>
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @author    Matt Schwab
 * @version   22 Mar 1997 
 */
public class CGNonlinSolver2 extends IterativeSlv implements Solver {

  protected Vector res;        //residual 
  protected float  rtrNew;     //norm2 of the residual

  //after each reset-th iteration I ensure conjugacy
  protected int    nReset = 50;

  //parameter for secant alphaUpdate
  protected float secantSigma = 0.1f;
  //parameter for general alphaUpdate
  protected int   nAlphaSteps = 10; 
  protected float alphaTol    = 0.00001f;

  public CGNonlinSolver2() {
    this(new IteratorSimple()); 
  }

  public CGNonlinSolver2(Iterator iterator) {
    this.iterator = iterator;
    this.iterator.setSolver(this); 
  }

  /**
   * after each nReset-th iteration I compute the exact residual to 
   * remove accumulated floating point error (default is 50).
   */
  public void setReset(int nReset) {
    this.nReset = nReset;
  }

  public Vector getResidual() {
    return res;
  }
  public float  getNormResidual() {
    return rtrNew;
  }

  public void solve(Operator A, Vector b, Vector x ) { 
    System.err.println("CGNonlinSolver2: " + 
                       "conversion to functional is not implemented yet!");
  }

  public void solve(hasGradient f, Vector x) { 
    if (! (f.getRange() instanceof RealSpc)) {
      System.err.println("Error in CGNonlinSolver2.solve(): " +
			 "operator f must be a functional");
    }

    this.A = f;
    this.x = x;
    this.b = new RealVec(0f);

    int iReset = 0; 

    res = A.getRange().newMember();
    res = ((hasGradient) A).getGradientAt(x).image(new RealVec(1F));
    res.neg();
    rtrNew   = res.norm2();

    Vector p = (Vector) res.copy();

    iterator.start();

    while(iterator.hasMoreIterations()) {

      //updates x
      alphaUpdate(p);

System.out.println("   Here is x:" + x);
      
      //updates r
      res = ((hasGradient) A).getGradientAt(x).image(new RealVec(1F));
      res.neg();

System.out.println("   Here is r:" + res);

      //updates p
      betaUpdate(p);

System.out.println("   Here is p:" + p); 

      iReset ++;
      if ( iReset >= nReset || res.dot(p) <= 0f ) {
	//System.out.println("I restart .......");
	// p.copy(res);
	p.zero();
	p.add(res);
	iReset = 0;
      }

      iterator.nextIteration(); 
      iter++;
    }
    iterator.finish();
  }

  protected void alphaUpdate(Vector p) {
    int    iAlphaSteps =   0;
    float  ptp      =   p.norm2();
    float  alpha    = - secantSigma; 
    Vector tmp      = A.getDomain().newMember();
    tmp.addScale(secantSigma, x, p);
    float  etaOld   = ((hasGradient) A).getGradientAt(tmp)
                                       .image(new RealVec(1F)).dot(p);
    do {
      float etaNew  = ((hasGradient) A).getGradientAt(x)
                                       .image(new RealVec(1F)).dot(p);
      alpha       *= etaNew / (etaOld - etaNew);
      //System.out.println("alpha is : " + alpha);
      x.addScale(alpha, x, p);
      etaOld       = etaNew;
      iAlphaSteps++;
    } while (iAlphaSteps < nAlphaSteps && alpha*ptp > alphaTol ); 
  } 

  protected void betaUpdate(Vector p) {
    float rtrOld = rtrNew;
    rtrNew       = res.norm2();
    float beta   = rtrNew/rtrOld;
System.out.println("beta is : " + beta);
    p.addScale( beta, res, p);
  }

}

