package juice.solver;

import  juice.example.*;
import  juice.vector.*;
import  juice.operator.*;
import  juice.util.Tester;
import  jam.solver.*;
import  jam.vector.*;
import  jam.operator.*;
import  java.util.Random;
      
/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
final
public class CGNonlinSolverTest2 {

  /**
   * I solve here 0 = F.image(x) for x = (x1, x2)
   * where F is the Rosenbrook's function 
   * F(x1,x2) = 100(x2-x1^2)^2 + (1- x1)^2
   * The unique minimum is x = (1,1). I use x = (-1.2, 1.0) 
   * as a starting value. 
   */ 
  public static void main( String[] args) {
    System.out.println( "CGNonlinSolver Test\n------------------\n");

    RnVec      x = new RnVec(2);
    int[] index0 = { 0 }; x.setVal(index0, -1.2f);
    int[] index1 = { 1 }; x.setVal(index1,  1.0f);

    RnVec      m = new RnVec(2);
    m.setVal(index0,  1.0f);
    m.setVal(index1,  1.0f);

    RealVec b = new RealVec(0f);

    RosenbrookOp F = new RosenbrookOp();
    LinearOperator fgrad = F.getGradientAt(x);
System.out.println("Here is fgrad(x): " + fgrad.image(new RealVec(1f)));

System.out.println("Here is F(x): " + F.image(x));
System.out.println("Here is F(m): " + F.image(m));

    CGNonlinSolver2 mysolver = new CGNonlinSolver2(); 
    mysolver.solve(F, x); 

System.out.println("Here is x: " + x);
System.out.println("Here is m: " + m);

    // e = |x - m| 
    RnVec    e = new RnVec((RnSpc) x.getSpace());
    e.sub(x, m);
    System.out.println("|X-x| = " + e.norm());
    Tester.exit(e.norm() < 10e-5);
  }
}

class RosenbrookOp implements hasGradient {
  RnSpc   domain;

  public RosenbrookOp()    { domain = new RnSpc(2); }
  public Space getDomain() { return domain; }
  public Space getRange()  { return RealSpc.getSpace(); }
  public jam.JamObject copy() { 
    System.err.println("RosenbrookOp: copy() is simply a clone"); 
    try {
      return (jam.JamObject) this.clone();
    }
    catch (Exception e) { System.err.println(e); }
    return null;
  }

  public void   image(Vector x, Vector d) {
    int[] index0 = {0}; float x1 = ((RnVec) x).getVal(index0); 
    int[] index1 = {1}; float x2 = ((RnVec) x).getVal(index1); 
    float tmp = 100f*(x2 - x1 * x1)*(x2 - x1*x1) + (1 - x1)*(1 - x1);
    ((RealVec) d).setValue(tmp); 
  }

  public Vector image(Vector x) {
    RealVec d = new RealVec();
    this.image(x, (Vector) d);
    return d;
  }
  public void addImage(Vector x, Vector d){ 
    d.add(this.image(x));
  }
  public void residual(Vector d, Vector b, Vector r) {
    r.sub(this.image(d),b);
  }
  public LinearOperator getGradientAt(Vector x) { 
    return new RosenbrookGradAt(x, (RnSpc) getDomain());
  }
}

class RosenbrookGradAt implements LinearOperator {
  RnSpc   range;
  Vector  x;

  RosenbrookGradAt(Vector x, RnSpc range) {
    this.range  = range;
    this.x      = x;
  }
  public Space 	       getDomain() { return RealSpc.getSpace(); }
  public Space 	       getRange () { return range; }
  public jam.JamObject copy() { 
    return new RosenbrookGradAt((Vector) x.copy(), range);
  }
  public void image(Vector real, Vector rn) {
    int[] index0 = {0}; float x1 = ((RnVec) x).getVal(index0); 
    int[] index1 = {1}; float x2 = ((RnVec) x).getVal(index1); 
    float f1 = -400f*x1*(x2 - x1 * x1) - 2*(1 - x1);
    float f2 =  200f*   (x2 - x1 * x1);
    ((RnVec) rn).setVal(index0, ((RealVec) real).getValue() * f1); 
    ((RnVec) rn).setVal(index1, ((RealVec) real).getValue() * f2); 
  }    
  public Vector image(Vector x) {
    RnVec d = new RnVec(2);
    this.image(x, (Vector) d);
    return d;
  }
  public void addImage(Vector x, Vector d){ 
    d.add(this.image(x));
  }
  public void residual(Vector d, Vector b, Vector r) {
    r.sub(this.image(d),b);
  }
}





