package juice.solver;

import  juice.example.*;
import  juice.operator.CompoundLinOpAdj;
import  jam.solver.*;
import  jam.vector.*;
import  jam.operator.*;
      
/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
final
public class CGPrecondSolverTest {

  /**
   * I solve here b = L x for x (b, L are known). 
   * I create the "true answer" m by filling it with random numbers. 
   * To ensure that L is positive definite I construct it as 
   * L = (A'A) where A is filled with random numbers. 
   * Next I set b = L m. 
   * Finally, I solve b = L x for x. I expect the solution to be x = m. 
   * As an initial guess I use x = 0. 
   */ 
  public static void main( String[] args) {
    System.out.println( "CGSolver Test\n------------\n");

    int[] mShape = { 5, 2 };
    RnVec      m = new RnVec(mShape);

    int[] dShape = { 2, 3, 4 };
    RnSpc dSpace = new RnSpc(dShape);

    RnOp  A      = new RnOp((RnSpc) m.getSpace(), dSpace);

    // set random values for m and A
    A.rand();
    m.rand();

    // L = A'A   (Normal equation as required by CGSolver) 
    CompoundLinOpAdj
      L = new CompoundLinOpAdj(A, A.adjoint());

    // b = A'Am = L m
    Vector   b =       L.image(m);    

    // CG solves b = L x. We expect to find x = m. Inital guess is x=0. 
    RnVec    x = new RnVec((RnSpc) m.getSpace());
    x.zero();


    // I build a preconditioning operator M. 
    // M is a diagonal matrix. Its diagonal elements are inverse 
    // to L's diagonal elements. 
    RnOp  M = new RnOp((RnSpc) b.getSpace());
    M.fill(0F);
    RnVec base = new RnVec((RnSpc) b.getSpace());

    int   dim       = ((RnSpc) b.getSpace()).size();
    int[] shape     = ((RnSpc) b.getSpace()).getShape();

    //System.out.println("dim   is: " + dim); 
    //System.out.println("shape is: " + shape[1]); 

    // Good place to use the enumeration for Rn
    // This is ugly!
    for (int i1 = 0; i1 < shape[0]; i1++) {
    for (int i2 = 0; i2 < shape[1]; i2++) {
      int[] index = {i1,i2};
      base.zero();
      base.setVal(index,1F);
      M.setVal(index,index,1/(base.dot(L.image(base))));
    }}

System.out.println("here is M: " + M);

    new CGPrecondSolver(M).solve( L, b, x );

System.out.println("here is x: " + x);
System.out.println("\n restart with x = 0 \n");
    x.zero();

    new CGSolver       ( ).solve( L, b, x );

System.out.println("here is x: " + x);

    // e = |x - m| 
    RnVec    e = new RnVec((RnSpc) x.getSpace());
    e.sub(x, m);
    System.out.println("|X-x| = " + e.norm());
    boolean passed = (e.norm() < 10e-5);
    System.out.println( passed ? "passed":"failed" );
    System.exit( passed ? 0:1 );

  }
}

