package juice.solver;

import  juice.example.*;
import  juice.operator.CompoundLinOpAdj;
import  jam.solver.*;
import  jam.vector.*;
import  jam.operator.*;
      
/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
final
public class CGSimpleTest {

  /**
   * I solve here b = L x for x (b, L are known). 
   * I create the "true answer" m by filling it with random numbers. 
   * To ensure that L is positive definite I construct it as 
   * L = (A'A) where A is filled with random numbers. 
   * Next I set b = L m. 
   * Finally, I solve b = L x for x. I expect the solution to be x = m. 
   * As an initial guess I use x = 0. 
   */ 
  public static void main( String[] args) {
    System.out.println( "CGSimple Test\n------------\n");

    int[] mShape = { 5, 2 };
    RnVec      m = new RnVec(mShape);

    int[] dShape = { 2, 3, 4 };
    RnSpc dSpace = new RnSpc(dShape);

    RnOp  A      = new RnOp((RnSpc) m.getSpace(), dSpace);

    // set random values for m and A
    A.rand();
    m.rand();

    // L = A'A   (Normal equation as required by CGSimple) 
    CompoundLinOpAdj
      L = new CompoundLinOpAdj(A, A.adjoint());

    // b = A'Am = L m
    Vector   b =       L.image(m);    

    // CG solves b = L x. We expect to find x = m. Inital guess is x=0. 
    RnVec    x = new RnVec((RnSpc) m.getSpace());
    x.zero();

    new CGSimple(25).solve( L, b, x );

    // e = |x - m| 
    RnVec    e = new RnVec((RnSpc) x.getSpace());
    e.sub(x, m);
    System.out.println("|X-x| = " + e.norm());
    boolean passed = (e.norm() < 10e-5);
    System.out.println( passed ? "passed":"failed" );
    System.exit( passed ? 0:1 );
  }
}

