package juice.solver;

import  jam.solver.*;
/**
 * <BR><!--#include file="./source.inc"--><BR>
 * @author      Matt Schwab 
 * @version     14 Mar 1997
 */
public class IteratorSimple implements ModularIterator {
  private Starter           starter;
  private Terminator        terminator;
  private Reporter          reporter;
  private Finisher          finisher;
  private IterativeSolver   solver;

  public IteratorSimple(Starter starter,  Terminator terminator,
		       Reporter reporter, Finisher   finisher) {
    this.starter    = starter;
    this.terminator = terminator;
    this.reporter   = reporter;
    this.finisher   = finisher;
  }

  public IteratorSimple() {
    this.starter    = new StarterSimple( );
    this.terminator = new TerminatorSimple( );
    this.reporter   = new ReporterEmpty();
    this.finisher   = new FinisherSimple( );
  }

  public IteratorSimple(float tol, int maxIter, boolean verbose) {
    this.starter = new StarterSimple( );
    this.terminator = new TerminatorSimple(maxIter, tol);
    if (verbose) this.reporter = new ReporterSimple();
    else         this.reporter = new ReporterEmpty();
    this.finisher   = new FinisherSimple( );
  }

  public void setStarter   (Starter         starter) {
    this.starter                          = starter;
  }
  public void setTerminator(Terminator      terminator) {
    this.terminator                       = terminator;
  }
  public void setReporter  (Reporter        reporter) {
    this.reporter                         = reporter;
  }
  public void setFinisher  (Finisher        finisher ) {
    this.finisher                         = finisher;
  }
  public void setSolver    (IterativeSolver solver) {
    this.solver                           = solver;
    starter.setSolver(solver);
    terminator.setSolver(solver);
    reporter.setSolver(solver);
    finisher.setSolver(solver);
  }

  public void    start() {
    starter.start();
  }
  public boolean hasMoreIterations() {
    return terminator.hasMoreIterations();
  }
  public void    nextIteration() {
    reporter.nextIteration();
  } 
  public void    finish() {
    finisher.finish();
  }
}

