package juice.solver;

import  juice.example.*;
import  juice.vector.*;
import  juice.operator.*;
import  jam.solver.*;
import  jam.vector.*;
import  jam.operator.*;
import  java.util.Random;
      
/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
final
public class SimAnnealSolverTest {

  /**
   * I solve here 0 = F.image(x) for x = (x1, x2)
   * where F is the Rosenbrock's function 
   * F(x1,x2) = 100(x2-x1^2)^2 + (1- x1)^2
   * The unique minimum is x = (1,1). I use x = (-1.2, 1.0) 
   * as a starting value. 
   * I would prefer a nonlinear problem with a few local minima 
   * but I cannot think of one right now. 
   */ 
  public static void main( String[] args) {
    System.out.println( "SimAnnealSolver Test\n------------------\n");

    int[] mShape = { 2 };
    RnVec      x = new RnVec(mShape);
    int[] index0  = { 0 }; x.setVal(index0, -1.2f);
    int[] index1  = { 1 }; x.setVal(index1,  1.0f);

    RnVec      m = new RnVec(mShape);
    m.setVal(index0,  1.0f);
    m.setVal(index1,  1.0f);

    RealVec b = new RealVec(0f);

    RosenbrookOp F = new RosenbrookOp((RnSpc) x.getSpace(), 
				      RealSpc.getSpace());

System.out.println("Here is F(x): " + F.image(x));
System.out.println("Here is F(m): " + F.image(m));

    RosenbrookProxyFactory proxyFactory = new RosenbrookProxyFactory();

    SimAnnealSolver metropolis = new SimAnnealSolver(proxyFactory);
    metropolis.solve(F, b, x); 

System.out.println("Here is x: " + x);
System.out.println("Here is m: " + m);

    // e = |x - m| 
    RnVec    e = new RnVec((RnSpc) x.getSpace());
    e.sub(x, m);
    System.out.println("|X-x| = " + e.norm());
    boolean passed = (e.norm() < 10e-5);
    System.out.println( passed ? "passed":"failed" );
    System.exit( passed ? 0:1 );
  }
}

class RosenbrookOp implements Operator {
  RnSpc   domain;
  RealSpc range; 
  RosenbrookOp(RnSpc domain, RealSpc range) {
    if (! ((domain.size() == 1) && (domain.getShape()[0] == 2))) {
      System.err.println("RosenbrookOp: domain has to be 2-D space");
    }
    this.domain = domain;
    this.range  = range;
  }
  public Space getDomain() { return domain; }
  public Space getRange( ) { return range;  }
  public jam.JamObject copy() { 
    System.err.println("RosenbrookOp: copy() is not implemented"); 
    return (jam.JamObject) this; 
  }
  public void image(Vector x, Vector d) {
    int[] index0 = {0}; float x1 = ((RnVec) x).getVal(index0); 
    int[] index1 = {1}; float x2 = ((RnVec) x).getVal(index1); 
    float tmp = 100f*(x2 - x1 * x1)*(x2 - x1*x1) + (1 - x1)*(1 - x1);
    ((RealVec) d).setValue(tmp); 
  }
  public Vector image(Vector x) {
    RealVec d = new RealVec();
    this.image(x, (Vector) d);
    return d;
  }
  public void addImage(Vector x, Vector d){ 
    d.add(this.image(x));
  }
  public void residual(Vector d, Vector b, Vector r) {
    r.sub(this.image(d),b);
  }
}

class RosenbrookProxyFactory implements ProxyFactory {
  int[] shape = {2};
  RnVec y = new RnVec(shape);
  Random random = new Random();

  RosenbrookProxyFactory() {}

  public Proxy createProxy(Operator A, Vector x, Vector b, float NormResidual){
    y.rand();
    RealVec res = new RealVec(0f);
    A.residual(y, b, res);
    return new RosenbrookProxy(res.norm(), (RnVec) y, (RnVec) x);
  }
  public float getEstimateOfNormResidual() {
    return 5f;
  }

} 

class RosenbrookProxy implements Proxy {
  float normResidual;
  RnVec y;
  RnVec x;
  RosenbrookProxy(float normResidual, RnVec y, RnVec x) {
    this.normResidual = normResidual;
    this.y            = y;
    this.x            = x;
  }
  public float  getNormResidual() {
    return normResidual;
  }
  public void setApproximation() {
    //x.copy(y);
    x.zero();
    x.add(y);
  }    
}



