package juice.solver;

import  juice.example.*;
import  juice.operator.*;
import  jam.solver.*;
import  jam.vector.*;
import  jam.operator.*;
import  java.util.Enumeration;
      
/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
final
public class SimplexSolverTest {

  /**
   * I solve here b = L x for x (b, L are known). 
   * I create the "true answer" m by filling it with random numbers. 
   * This is probably not necessary, but 
   * to ensure that L is positive definite I construct it as 
   * L = (A'A) where A is filled with random numbers. 
   * Next I set 0 = F(x) = |L x - b|^2 which creates the functional F(x). 
   * Finally, I solve F(x) = 0  for x. I expect the solution to be x = m. 
   * As an initial guess I use x = 0. 
   */ 
  public static void main( String[] args) {
    System.out.println( "SimplexSolver Test\n------------------\n");

    int[] mShape = { 5, 2 };
    RnVec      m = new RnVec(mShape);
    m.rand();

    int[] dShape = { 2, 3, 4 };
    RnSpc dSpace = new RnSpc(dShape);
    RnOp  A      = new RnOp((RnSpc) m.getSpace(), dSpace);
    A.rand();

    // L = A'A   (Normal equation as required by CGSolver) 
    CompoundLinOpAdj
      L = new CompoundLinOpAdj(A, A.adjoint());

    // b = A'Am = L m
    Vector   b =       L.image(m);    

    // Inital guess is x=0. 
    RnVec    x = new RnVec((RnSpc) m.getSpace());
    x.zero();

    // HERE IS WHERE AN APPLICATION WOULD START
    // build the basis functions: for loop for basisFct
    int   dim       = ((RnSpc) x.getSpace()).size();
    int[] shape     = ((RnSpc) x.getSpace()).getShape();
    RnVec   [] xbase = new RnVec   [dim]; 

    //System.out.println("dim   is: " + dim); 
    //System.out.println("shape is: " + shape[1]); 

    // Good place to use the enumeration for Rn
    // This is ugly!
    for (int i1 = 0; i1 < shape[0]; i1++) {
    for (int i2 = 0; i2 < shape[1]; i2++) {
      int[] index2 = {i1,i2};
      int   index1 = i1 + i2 * shape[0];
      xbase[index1] = new RnVec((RnSpc) x.getSpace());
      xbase[index1].setVal(index2,1F);
      //System.out.println("xbase: " + index1 + xbase[index1]);
    }}

    SimplexSolver simply = new SimplexSolver(xbase);

    // call solve routine solve(Operator L, Vector b, initial guess x)

    simply.solve(L, b, x); 

System.out.println("Here is x: " + x);
System.out.println("Here is m: " + m);

    // compare solution x with m (as before in CG stuff)

    /* Old CG stuff ... 
    new CGSolve((float) 1.0e-5, 50, true).solve( L, b, x );

    // e = |x - m| 
    RnVec    e = new RnVec((RnSpc) x.getSpace());
    e.sub(x, m);
    System.out.println("|X-x| = " + e.norm());
    boolean passed = (e.norm() < 10e-5);
    System.out.println( passed ? "passed":"failed" );
    System.exit( passed ? 0:1 );
    */
  }
}


