package juice.solver;

import  juice.example.*;
import  juice.operator.CompoundLinOpAdj;
import  jam.solver.*;
import  jam.vector.*;
import  jam.operator.*;
      
/**
 * <BR><!--#include file="./source.inc"--><BR>
 */
final
public class SteepDescentSlvTest {

  /**
   * I solve here b = L x for x (b, L are known). 
   * I create the "true answer" m by filling it with random numbers. 
   * To ensure that L is positive definite I construct it as 
   * L = (A'A) where A is filled with random numbers. 
   * Next I set b = L m. 
   * Finally, I solve b = L x for x. I expect the solution to be x = m. 
   * As an initial guess I use x = 0. 
   */ 
  public static void main( String[] args) {
    System.out.println( "SteepDescentSlv Test\n------------\n");

    int[] mShape = { 5, 2 };
    RnVec      m = new RnVec(mShape);

    int[] dShape = { 2, 3, 4 };
    RnSpc dSpace = new RnSpc(dShape);

    RnOp  A      = new RnOp((RnSpc) m.getSpace(), dSpace);

    // set random values for m and A
    A.rand();
    m.rand();

    // L = A'A   (Normal equation as required by SteepDescentSlv) 
    CompoundLinOpAdj
      L = new CompoundLinOpAdj(A, A.adjoint());

    // b = A'Am = L m
    Vector   b =       L.image(m);    

    // SteepDescent solves b = L x. We expect to find x = m. Inital guess is x=0. 
    RnVec    x = new RnVec((RnSpc) m.getSpace());
    x.zero();

    new SteepDescentSlv().solve( L, b, x );

    // e = |x - m| 
    RnVec    e = new RnVec((RnSpc) x.getSpace());
    e.sub(x, m);
    System.out.println("|X-x| = " + e.norm());
    boolean passed = (e.norm() < 10e-5);
    System.out.println( passed ? "passed":"failed" );
    System.exit( passed ? 0:1 );
  }
}



// **************************************************
// Here is the old version using Rsf's and Icai
/*
final
public class SteepDescentSlvTest {
  public static void main( String[] args) {
    System.out.println( "SteepDescentSlv Test\n------------\n");

    Axis[] inAx = new Axis[2];
    inAx[0] = new Axis( 5, 1f, 0f, "time", "s" );
    inAx[1] = new Axis( 6, 2f, 0f, "distance", "m" );

    Axis[] ffAx = new Axis[2];
    ffAx[0] = new Axis( 3, 1f, 0f, "", "" );
    ffAx[1] = new Axis( 3, 1f, 0f, "", "" );

    Rsf inRsf = new Rsf(inAx);
    inRsf.rand();
    Rsf ffRsf = new Rsf(ffAx);
    ffRsf.rand();

    Icai myIcai = new Icai( ffRsf, "input", (RsfSpace) inRsf.getSpace() );

    CompoundLinearOperatorAdjoint
      L = new CompoundLinearOperatorAdjoint( "normal", myIcai);

    Vector b =       L.image( inRsf);
    Rsf    x = (Rsf) L.getDomain().newMember();
    x.rand();

    new SteepDescentSlv().solve( L, b, x );
    System.out.println( "b:\n" + b + "\n" +
			"inRsf:\n" + inRsf +
			"x:\n" + x );
    x.sub( inRsf);
    System.out.println( "X - x\n" + x);
  }
}
*/
