package juice.util;

import  jam.vector.*;
import  jam.operator.*;

/**
 * JuiceUtil contains a set of utility functions implemented as class methods.
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @author Matt Schwab, Joel Schroeder
 */
public class JuiceUtil {

  public static float dotProductTest(hasAdjoint FWD,
				     Vector w, Vector v,
				     String verbose) {
    return dotProductTest(FWD, FWD.adjoint(), w, v, verbose );
  }

  public static float dotProductTest(LinearOperator FWD,
				     LinearOperator ADJ,
				     Vector w, Vector v,
				     String verbose ) {
    boolean vrb = verbose.equals("verbose");
    if (vrb) {
      System.out.print( "DOT PRODUCT TEST" 	  + "\n" +
			"----------------" 	  + "\n" +
			"Operator (FWD):\n" + FWD + "\n" +
			"Operator (ADJ):\n" + ADJ + "\n" +
			"Vector 1 (w):\n"   + w   + "\n" +
			"Vector 2 (v):\n"   + v   + "\n" ); }
    
    Vector FWDw = FWD.image(w);
    Vector ADJv = ADJ.image(v);
    
    float vFWDw = v.dot(FWDw);
    float wADJv = w.dot(ADJv);
    float diff  = vFWDw - wADJv ;

    System.out.print( "\n" +
		      "RESULTS:"               + "\n" +
		      "<v, FWD(w)> = " + vFWDw + "\n" +
		      "<w, ADJ(v)> = " + wADJv + "\n" +
		      "Difference = "  + diff  + "\n" );
    return diff;
  }

  public static float[]   arrayfill(int dim, float   val) {
    float  [] xarray = new float  [dim];
    for (int i=0; i<dim; i++) 
      xarray[i] = val;
    return xarray;
  }
  public static int[]     arrayfill(int dim, int     val) {
    int    [] xarray = new int    [dim];
    for (int i=0; i<dim; i++) 
      xarray[i] = val;
    return xarray;
  }
  public static double[]  arrayfill(int dim, double  val) {
    double [] xarray = new double [dim];
    for (int i=0; i<dim; i++) 
      xarray[i] = val;
    return xarray;
  }
  public static boolean[] arrayfill(int dim, boolean val) {
    boolean[] xarray = new boolean[dim];
    for (int i=0; i<dim; i++) 
      xarray[i] = val;
    return xarray;
  }

  public static String  arrayToString( String lbl, Object[] arr,
				       String separator ) {
    StringBuffer buf = new StringBuffer();
    for ( int i=0 ; i<arr.length ; i++ ) {
      buf.append( lbl + "[" );
      buf.append( i );
      buf.append( "]\n" + arr[i].toString() + separator ); }
    return buf.toString(); }
  public static String  arrayToString( float [] arr,
				       String separator ) {
    StringBuffer buf = new StringBuffer();
    for ( int i=0 ; i<arr.length-1; i++ ) {
      buf.append( arr[i] ); buf.append( separator ); }
    buf.append( arr[ arr.length-1]);
    return buf.toString(); }

  /** separator1 separates the elements; separator2 separates the 1-D arrays */
  public static String  arrayToString( float [][] arr,
				       String separator1,
				       String separator2) {
    StringBuffer buf = new StringBuffer();
    for ( int i=0 ; i<arr.length-1; i++ ) {
      buf.append(JuiceUtil.arrayToString(arr[i],separator1)); 
      buf.append(separator2); 
    }
    buf.append(JuiceUtil.arrayToString(arr[arr.length-1],separator1));
    return buf.toString(); 
  }
  /** 
   * separator1 separates the elements; 
   * separator2 separates the 1-D arrays 
   * separator3 separates the 2-D arrays 
   */
  public static String  arrayToString( float [][][] arr,
				       String separator1,
				       String separator2,
				       String separator3) {
    StringBuffer buf = new StringBuffer();
    for ( int i=0 ; i<arr.length-1; i++ ) {
      buf.append(JuiceUtil.arrayToString(arr[i],separator1,separator2)); 
      buf.append( separator3 ); 
    }
    buf.append(JuiceUtil.arrayToString(arr[arr.length-1],separator1,
				                         separator2));
    return buf.toString(); 
  }

  public static String  arrayToString( int   [] arr,
				       String separator ) {
    StringBuffer buf = new StringBuffer();
    for ( int i=0 ; i<arr.length-1; i++ ) {
      buf.append( arr[i] ); buf.append( separator ); }
    buf.append( arr[ arr.length-1]);
    return buf.toString(); }
  public static String  arrayToString( boolean [] arr,
				       String  separator ) {
    StringBuffer buf = new StringBuffer();
    for ( int i=0 ; i<arr.length-1; i++ ) {
      buf.append( arr[i] == true ? "0":"1"); buf.append( separator ); }
    buf.append( arr[ arr.length-1]);
    return buf.toString(); }

  public static boolean equalArrays( Object[] objs1, Object[] objs2) {
    if (objs1 == objs2) return true;
    if (objs1 == null || objs2 == null) return false;
    int n  = objs1.length;
    if (n != objs2.length) return false;
    for (int i = 0; i < n; i++)
      if ( !(objs1[i].equals(objs2[i]))) return false;
    return true;
  }
  public static boolean equalArrays( float[] flts1, float[] flts2) {
    if (flts1 == flts2) return true;
    if (flts1 == null || flts2 == null) return false;
    int n  = flts1.length;
    if (n != flts2.length) return false;
    for (int i = 0; i < n; i++)
      if ( flts1[i] != flts2[i] ) return false;
    return true;
  }
  public static boolean equalArrays( int[] ints1, int[] ints2) {
    if (ints1 == ints2) return true;
    if (ints1 == null || ints2 == null) return false;
    int n  = ints1.length;
    if (n != ints2.length) return false;
    for (int i = 0; i < n; i++)
      if ( ints1[i] != ints2[i] ) return false;
    return true;
  }
  public static boolean equalArrays( boolean[] bool1, boolean[] bool2) {
    if (bool1 == bool2) return true;
    if (bool1 == null || bool2 == null) return false;
    int n  = bool1.length;
    if (n != bool2.length) return false;
    for (int i = 0; i < n; i++)
      if ( bool1[i] != bool2[i] ) return false;
    return true;
  }

  // ******** java.util.approx *********/

  public static boolean approx(double f1, double f2, double eps) {
    if (eps < 0.f) System.err.println("Error: wrong argument. Eps < 0");
    return (((f1 + eps) > f2) && ((f1 - eps) < f2));
  } 
  public static boolean approx(double f1, double f2) {
    double eps = Math.min(Math.abs(f1),Math.abs(f2)); 
    eps = Math.max(eps/1000., Float.MIN_VALUE);
    return JuiceUtil.approx(f1, f2, eps);
  }
  public static boolean approx(float f1, float f2, float eps) {
    return JuiceUtil.approx((double) f1, (double) f2, (double) eps);
  } 
  public static boolean approx(float f1, float f2) {
    return JuiceUtil.approx((double) f1, (double) f2);
  }

  // ******** java.util.Vector *********/

  // I need "java.util.Vector d" to avoid mix up with jam.vector.Vector
  public static boolean equalVectors(java.util.Vector d, 
					java.util.Vector e) { 
    if (d == null || e == null) return false;
    if (d.size()  != e.size() ) return false;
    // this could probably use enumeration
    for (int i = 0; i < d.size(); i++) 
      if (! (d.elementAt(i).equals(e.elementAt(i)))) return false;
    return true; 
  } 
  public static String vectorToString(String lbl, java.util.Vector v, 
				      String separator) {
    StringBuffer buf = new StringBuffer();
    for (int i=0; i < v.size(); i++) {
      buf.append( lbl + "[" + i + "]\n" );
      try                            { buf.append(v.elementAt(i)         ); }
      catch (NullPointerException e) { buf.append( i + "-th axis not set"); }
      buf.append( separator ); 
    }
    return buf.toString();
  }
}
