package juice.util;

/**
 * This class implements standard exit routines for Test classes.<BR>
 * Ideally, every class (e.g. Rsf) will have a corresponding Test
 * class (e.g. RsfTest) which has a main routine which does a few
 * tests to see if the class is working correctly.  These tests can also
 * be used as examples of how to use these classes.<BR><BR>
 *
 * Test objects should be able to be run as a main() routine, which
 * will exit with a return value to the shell, and with a callable
 * routine, which executes and returns the value without exiting.<BR>
 * <BR>Future work:<BR>
 * We probably want to enforce consistency by creating an interface or
 * abstract class for Test routines.
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class Tester {
  /**
   * Prints "passed" if passed==true, otherwise prints "failed".
   * Exits with value 0,1 if passed==true,false, respectively.
   */
  public static void exit(boolean passed) {
    System.out.println(passed ? "passed":"failed");
    System.exit(passed ? 0:1);
  }
  /**
   * Prints "passed" if passed==0, otherwise prints "failed".
   * Exits with value of passed
   */
  public static void exit(int     passed) {
    System.out.println((passed==0) ? "passed":"failed");
    System.exit(passed);
  }
  /** 
   * Compares two floats if they are approximately equally. 
   * Returns true if they are, false if they are not. 
   * Prints a statement to System.out about the comparison. 
   */
  public static boolean isApprox(String var,float f1,float f2) {
    System.out.println("Variable " + var + " should be " + f1 + ": " + f2);
    return JuiceUtil.approx(f1,f2);
  }
}
