package juice.vector;

import    jam.JamObject;
import    jam.vector.*;
import   java.util.Enumeration;
import   java.util.NoSuchElementException;

/**
 * <BR><!--#include file="./source.inc"-->
 *
 * @author      Joel Schroeder
 */
public class ProductVec implements ProductVector {
  ProductSpace     space;
  java.util.Vector vecList;

  protected ProductVec(ProductSpace space) {
    this.space   = space;
    this.vecList = new java.util.Vector(space.size());
    this.vecList.setSize(space.size());
  }

  public ProductVec(Vector vec0, Vector vec1) {
    space   = new ProductSpc(vec0.getSpace(), vec1.getSpace());
    vecList = new java.util.Vector(2);
    vecList.setSize(2);
    setSubVector(0, vec0);
    setSubVector(1, vec1);
  }

  public JamObject copy() {
    ProductVec cp = new ProductVec(this.space);
    for (int i = 0; i < size(); i++)
      cp.setSubVector(i, (Vector) this.getSubVector(i).copy());
    return cp;
  }

  public ProductVec(ProductVec src) {
    this(src.space);
    for (int i = 0; i < size(); i++) {
      Vector subVec = src.getSubVector(i);
      this.setSubVector(i, (subVec == null) ? null:((Vector) subVec.copy()));
    }
  }

  public Space     getSpace() { return space; }

  public void      neg     () {
    for (int i = 0; i < space.size(); i++)
      this.getSubVector(i).neg();
  }
  public void      neg     (Vector vec2) {
    for (int i = 0; i < space.size(); i++)
      this.getSubVector(i).neg(((ProductVector) vec2).getSubVector(i));
  }
  public void      add     (Vector vec2) {
    for (int i = 0; i < space.size(); i++)
      this.getSubVector(i).add(((ProductVector) vec2).getSubVector(i));
  }
  public void      add     (Vector vec1,
			    Vector vec2) {
    for (int i = 0; i < space.size(); i++)
      this.getSubVector(i).add(((ProductVector) vec1).getSubVector(i),
			       ((ProductVector) vec2).getSubVector(i));
  }
  public void      addScale(float a,
			    Vector vec2) {
    for (int i = 0; i < space.size(); i++)
      this.getSubVector(i).
	addScale(a, ((ProductVector) vec2).getSubVector(i));
  }
  public void      addScale(float a,
			    Vector vec1,
			    Vector vec2) {
    for (int i = 0; i < space.size(); i++)
      this.getSubVector(i).
	addScale(a,
		 ((ProductVector) vec1).getSubVector(i),
		 ((ProductVector) vec2).getSubVector(i));
  }
  public void      sub     (Vector vec2) {
    for (int i = 0; i < space.size(); i++)
      this.getSubVector(i).sub(((ProductVector) vec2).getSubVector(i));
  }
  public void      sub     (Vector vec1,
			    Vector vec2) {
    for (int i = 0; i < space.size(); i++)
      this.getSubVector(i).sub(((ProductVector) vec1).getSubVector(i),
			       ((ProductVector) vec2).getSubVector(i));
  }
  public void      scale   (float  a   ) {
    for (int i = 0; i < space.size(); i++)
      this.getSubVector(i).scale(a);
  }
  public void      scale   (float  a   ,
			    Vector vec ) {
    for (int i = 0; i < space.size(); i++)
      this.getSubVector(i).scale(a,
				 ((ProductVector) vec).getSubVector(i));
  }
  public void      copyFrom(Vector src) {
    for (int i = 0; i < space.size(); i++)
      this.getSubVector(i).copyFrom(((ProductVector) src).getSubVector(i));
  }
  public void      zero    () {
    for (int i = 0; i < space.size(); i++)
      this.getSubVector(i).zero ();
  }

  public float     dot     (Vector vec2) {
    float sum = 0F;
    for (int i = 0; i < space.size(); i++)
      sum += this.getSubVector(i).
	dot(((ProductVector) vec2).getSubVector(i));
    return sum;
  }

  public float     norm2   () {
    float sum = 0F;
    for (int i = 0; i < space.size(); i++)
      sum += this.getSubVector(i).norm2();
    return sum;
  }

  public float     norm    () { return (float) Math.sqrt(norm2()); }

  public Vector getSubVector(int i) { return (Vector) vecList.elementAt(i); }
  public void   setSubVector(int i, Vector vec) {
    if (! vec.getSpace().equals(space.getSubSpace(i)))
      System.err.println("Error in ProductVec.setSubVector():\n" +
			 "\tspace of this incompatible with that of " +
			 "argument vec\n");
    vecList.setElementAt(vec, i);
  }

  public Enumeration  elements() {
    return new ProductVecEnumerator(this);
  }

  /**
   * @return number of vectors in the product vector
   */
  public int   size () { return space.size(); }

  public String toString() {
    return("ProductVec: toString() not yet implemented\n");
  }
}

  // ProductVecEnumerator is based on the source code for
  // java.util.VectorEnumerator
final
class ProductVecEnumerator implements Enumeration {
  java.util.Vector vecList;
  int              count;

  ProductVecEnumerator(ProductVec productVector) {
    this.vecList  = productVector.vecList;
    this.count    = 0;
  }

  public boolean hasMoreElements() { return count < vecList.size(); }

  public Object  nextElement    () {
    synchronized (vecList) {
      if (count < vecList.size())
	return    vecList.elementAt(count++);
    }
    throw new NoSuchElementException("ProductVecEnumerator");
  }
}
