package juice.vector;

import    jam.vector.*;
import    jam.JamObject;

/**
 * This class has no public constructors.  Use the static method
 * <TT>getSpace()</TT> to get an instance of RealSpc.<BR>
 *
 * <BR><!--#include file="./source.inc"-->
 *
 * @author      Joel Schroeder
 */
public class RealSpc implements RealSpace {
  private static RealSpc reals = null;

  /**
   * This acts as the constructor for <TT>RealSpc</TT>.<BR>
   * Because every instance of RealSpc represents the same mathematical space,
   * it makes sense to allow at most one instance of RealSpc to be in
   * existence at a time.<BR>
   * This static method can enforce that rule.
   */
  public static RealSpc getSpace() {
    if (reals == null) reals = new RealSpc();
    return reals;
  }

  private RealSpc() {}

  public Vector    newMember()         { return new RealVec(); }
  public boolean   hasMember(Vector v) { return this.equals(v.getSpace()); }

  /**
   * Since there is at most only one instance of a RealSpc, this must be a
   * shallow copy.
   */
  public JamObject copy() { return this; }

  public boolean equals(Object obj)  {
    if (this == obj) return true;
    return (obj instanceof RealSpace);
  }
  public String toString() {
    return "juice.vector.RealSpc";
  }

}
